% Ce programme gnre des couples (\theta_1,\theta_2) distribus suivant la
% loi a posteriori N(m,Sigma) avec
% m=(y_1,y_2)
% Sigma = [1 rho; rho 1];

% par chantillonnage de Gibbs
% \theta_1 | \theta_2,y \sim N(y_1+rho*(theta_2-y_2),1-rho^2)
% \theta_2 | \theta_1,y \sim N(y_2+rho*(theta_1-y_1),1-rho^2)

% ou par Metropolis within Gibbs supposant que la loi de \theta_2 |
% \theta_1,y n'est pas connue. On simule alors suivant une loi de
% proposition et acceptation rejet
clear
y_1 = 0; y_2 = 0;
rho = .8;
M = 10000;
n0=1000;
th = zeros(M,2);
th(1,1)=2.5; th(1,2)=-2.5;
for ii=2:M+n0
  
 % Gnration suivant la loi conditionnelle de \theta_1 | \theta_2,y
 th(ii,1)=y_1+rho*(th(ii-1,2)-y_2)+sqrt(1-rho^2)*randn(1,1);
  
   % Metropolis within Gibbs
   % gnration d'un candidat N(th(ii-1,2), 2), i.e. marche alatoire de
   % variance 2
   thstar2 = th(ii-1,2)+2*randn(1,1);
   thold=th(ii-1,2);
   % loi conditionnelle de \theta_2 | \theta_1,y value en th(ii-1,2)
   mu=y_2+rho*(th(ii,1)-y_1);
   pold  = (2*pi)^(-0.5)*(1-rho^2)^(-1/2)*exp(-(thold -mu)^2/(2*(1-rho^2)));
   % loi conditionnelle de \theta_2 | \theta_1,y value en thstar2
      pstar = (2*pi)^(-0.5)*(1-rho^2)^(-1/2)*exp(-(thstar2 -mu)^2/(2*(1-rho^2)));
   % Loi de proposition value en th(ii-1,2)
    qold = normpdf(th(ii-1,2),th(ii-1,2),2);
   % Loi de proposition value en thstar2
   qstar = normpdf(thstar2,th(ii-1,2),2);
   % probabilit d'acceptation
   accept = min(pstar*qold/(pold*qstar),1);
   % tirage uniforme sur [0,1]
   urn = rand(1);
   % Procdure d'acceptation rejet
   th(ii,2) = thstar2*(urn<=accept)+th(ii-1,2)*(urn>accept);
   % dans le cas de Gibbs, ajouter la ligne ci-dessous
   th(ii,2)=y_2+rho*(th(ii,1)-y_1)+sqrt(1-rho^2)*randn(1,1);
end;

figure(1)
scatter(th(n0+1:end,1),th(n0+1:end,2),'filled')
xlabel('\theta_1','fontsize',14)
ylabel('\theta_2','fontsize',14)

figure(3)
histfit(th(n0+1:end,1))

figure(4)
histfit(th(n0+1:end,2))

% estimation de la covariance
disp('moyennes')
mean(th(n0+1:end,1))
mean(th(n0+1:end,2))
disp('variances')
var(th(n0+1:end,1))
var(th(n0+1:end,2))
disp('covariance')
sum(th(n0+1:end,1).*th(n0+1:end,2))/(M)-mean(th(n0+1:end,1))*mean(th(n0+1:end,2))
